% Stanford, 2/11/2014
% -------------------------------------------------------------------
% demoRieszTextureLearning2D.m
% 
% Demo script for Riesz multiscale texture learning
% IMPORTANT: It requires libsvm wrapper for matlab
% -------------------------------------------------------------------
%
% Adrien Depeursinge, adepeurs@stanford.edu
% Stanford University, CA, USA
%
% -------------------------------------------------------------------
%
% REFERENCES:
% [1] A. Depeursinge, A. Foncubierta-Rodriguez, D. Van De Ville, H. Mller, 
%       "Rotation-covariant texture learning using steerable Riesz wavelets",
%       in: IEEE Transactions on Image Processing, 23:2(898-908), 2014.
%
% [2] A. Depeursinge, A. Foncubierta-Rodriguez, D. Van De Ville, H. Mller,
%       "Multiscale Lung Texture Signature Learning Using The Riesz Transform",
%       in: Medical Image Computing and Computer-Assisted Intervention 
%       MICCAI 2012, Nice, France, pages 517-524, Springer Berlin / Heidelberg, 2012. 
%
% [3] M. Unser, D. Van De Ville, N. Chenouard
%       "Steerable Pyramids and Tight Wavelet Frames in L2(Rd)",
%       in: IEEE Transactions on Image Processing, 20:10(2705-2721), 2011.
%
% -------------------------------------------------------------------

clc;close all;
%clear all;
imageSize=128;
[X,Y] = meshgrid(1:imageSize,1:imageSize);

% Create toy images I1 and I2
I1=sin(X/(imageSize/64))+sin(Y/(imageSize/64))+rand(imageSize); % checkerboard + noise
% I1=sin((0.5*X+0.5*Y)/(imageSize/128))+rand(imageSize); % lines + noise
% I1 = sin(X/(imageSize/96)+5*sin(Y/(imageSize/16)))+sin(Y/(imageSize/96))+rand(imageSize); % wiggled checkerboard + noise

I2=zeros(imageSize)+rand(imageSize);

subplot(1,2,1);imshow(I1,[]);title('texture1');
subplot(1,2,2);imshow(I2,[]);title('texture2');

N=2;     % Riesz order
J=4;      % number of decomposition levels

% compute Riesz coefficients
rieszCoeffs1=RieszTextureAnalysis(I1,N,J,false,true);
% viewRieszCoefficient(rieszCoeffs1);
rieszCoeffs2=RieszTextureAnalysis(I2,N,J,false,true);
features1=rieszEnergiesInMask(rieszCoeffs1,ones(size(I1)),true);
features2=rieszEnergiesInMask(rieszCoeffs2,ones(size(I2)),true);

% compute the signature: 
% Eq. (9) of [1]
w=GetSVMweightsFromCoeffs(features1,features2); 

% visualize the texture signatures for every scale
countSubplot=1;
figure;set(gcf, 'units','normalized','outerposition',[0 0 1 1]);%fullscreen
for iterScale=1:N+1:J*(N+1),
    fig=subplot(1,J,countSubplot);
    wScale=w(iterScale:iterScale+N);
    RieszViewTemplate(wScale,N,fig,['scale ' num2str(countSubplot) ', weight: ' num2str(mean(abs(wScale)),'%10.2e\n')]);
    countSubplot=countSubplot+1;
end;
suptitle('texture1 versus texture2');

% now maximize the local magnitude of the signature w at the smallest scale:
% Eq. (11) of [1], yields (N+1)*J features
learnedFeatures1=alignCoefficients_1stScaleEnergy(rieszCoeffs1,w,N,J);
learnedFeatures2=alignCoefficients_1stScaleEnergy(rieszCoeffs2,w,N,J);

% an alternative is to use the energy of the maximum magnitude of the
% signature w for each scale: yields J features
learnedFeaturesMaxMagnitude1=alignCoefficients_MaxMagnitudeEnergy(rieszCoeffs1,w,N,J);
learnedFeaturesMaxMagnitude2=alignCoefficients_MaxMagnitudeEnergy(rieszCoeffs2,w,N,J);
